library example.jwt_cookie.server;

import 'dart:async';
import 'dart:io';
import 'dart:convert';

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_session/jaguar_session.dart';
import 'package:jaguar_jwt/jaguar_jwt.dart';
import 'package:dart_jwt/dart_jwt.dart';

import '../../common/models/book/book.dart';
import '../../common/user.dart';
import '../../common/books.dart';

part 'server.g.dart';

/// This route group contains login and logout authentication routes
@RouteGroup()
//You must provide an instance of SessionManager through sessionManager parameter
@Wrap(const [#sessionInterceptor])
class AuthRoutes {
  @Post(path: '/login')
  //BasicAuth interceptors uses SessionManager provided by SessionInterceptor
  //to create a new session if authentication is successful
  @WrapBasicAuth(kModelManager, authorizationIdKey: 'sub')
  void login(HttpRequest req) {}

  @Post(path: '/logout')
  @Input(SessionInterceptor)
  Future<Null> logout(SessionManager sessionManager) async {
    await sessionManager.clearSession();
  }

  WrapSessionInterceptor sessionInterceptor() =>
      new WrapSessionInterceptor(sessionManager());

  //This is the session manager that is passed to SessionInterceptor
  JwtCookieSessionManager sessionManager() => new JwtCookieSessionManager(
      _kJwtAuthenticationCtx, _kJwtAuthorizationCtx);

  ///JWT Authentication context used by [JwtAuthHeaderSessionManager]
  final _kJwtAuthenticationCtx = new JwtAuthenticationContext(
      new JwaSymmetricKeySignatureContext(
          'sdgdflgujsdgndsflkgjsdlnwertwert78676'),
      'jaguar',
      audience: ['book']);

  ///JWT Authorization context used by [JwtAuthHeaderSessionManager]
  final _kJwtAuthorizationCtx = new JwtAuthorizationContext(
      new JwaSymmetricKeySignatureContext(
          'sdgdflgujsdgndsflkgjsdlnwertwert78676'),
      'jaguar',
      audience: ['admin']);
}

/// Collection of routes that need authorization
@RouteGroup(path: '/book')
//You must provide an instance of SessionManager through sessionManager parameter
@Wrap(const [#sessionInterceptor])
//UserAuthoriser interceptor uses SessionManager provided by SessionInterceptor
//to parse and validate session information. UserAuthoriser interceptor provides
//logged-in user as Input to other interceptors and routes.
@WrapUserAuthoriser(kModelManager, sessionIdKey: 'sub')
class StudentRoutes {
  @Get(path: '/all')
  //loggedinUser can be obtained from UserAuthoriser as Input
  @Input(UserAuthoriser)
  Response<String> getAllBooks(User loggedinUser) {
    List<Map> ret = books.values.map((Book book) => book.toMap()).toList();

    Response<String> resp = new Response<String>(JSON.encode(ret));
    //For demonstration we pass username of logged-in user as cookie
    resp.cookies.add(new Cookie('user', loggedinUser.username));
    return resp;
  }

  @Get(path: '/:id', headers: const {'sample-header': 'sample-value'})
  String getBook(String id) {
    Book book = books[id];
    return book.toJson();
  }

  WrapSessionInterceptor sessionInterceptor() =>
      new WrapSessionInterceptor(sessionManager());

  //This is the session manager that is passed to SessionInterceptor
  //We dont need Authentication context for authorization, so it is set to null
  JwtCookieSessionManager sessionManager() =>
      new JwtCookieSessionManager(null, _kJwtAuthorizationCtx);

  ///JWT Authorization context used by [JwtAuthHeaderSessionManager]
  final _kJwtAuthorizationCtx = new JwtAuthorizationContext(
      new JwaSymmetricKeySignatureContext(
          'sdgdflgujsdgndsflkgjsdlnwertwert78676'),
      'jaguar',
      audience: ['book']);
}

@Api(path: '/api')
class LibraryApi {
  @Group()
  final AuthRoutes auth = new AuthRoutes();

  @Group()
  final StudentRoutes student = new StudentRoutes();
}

server() async {
  final api = new JaguarLibraryApi();

  Jaguar jaguar = new Jaguar();
  jaguar.addApi(api);

  await jaguar.serve();
}
